--- 模块功能：串口功能
-- @author openLuat
-- @module uartOut
-- @license MIT
-- @copyright openLuat
-- @release 2018.03.27

module(...,package.seeall)

require"utils"
require"pm"
require"ril"
require"http"

local urlSent = false

local function cbFnc(result,prompt,head,body)
    log.info("Http.cbFnc",result,prompt)
    sys.publish(result,body)
end


local function encodeUrl(s)
    local body = string.char(0xe7,0x7e,0x0f)..s
    local check = body:byte()
    for i=2,body:len() do
        check = bit.bxor(check,body:byte(i))
    end
    return body..string.char(check)
end

sys.taskInit(function ()
    sys.waitUntil("IP_READY_IND")
    while true do
        http.request("GET","http://wxapp.mafu.shop/openapi/qrcode/fetch?imei="..misc.getImei(),nil,nil,nil,30000,httpcb)
        local _,r,d = sys.waitUntil("HTTP_GET_OK")
        if r then
            local tjsondata,result,errinfo = json.decode(d)
            if result and type(tjsondata)=="table" then
                if tjsondata.data and tjsondata.data.url then
                    while not urlSent do
                        --发串口数据
                        write(encodeUrl(tjsondata.data.url))
                        sys.wait(500)
                    end
                end
            end
        end
    end
end)

--串口ID
local UART_ID = _G.moduleType == 2 and 1 or 2
--串口读到的数据缓冲区
local rdbuf = ""

--待接收的数据
local receiveData
--设置待接收数据
function setReceiveData(s)
    receiveData = s
end

local collect
--开始缓存数据
function startCollect()
    rdbuf = ""
    collect = true
end
--停止缓存数据
function stopCollect()
    collect = false
    local r = rdbuf
    log.info("uart.stopCollect",rdbuf:toHex(),r:toHex())
    rdbuf = ""
    return r
end

local lock = false
--[[
函数名：proc
功能  ：处理从串口读到的数据
参数  ：
        data：当前一次从串口读到的数据
返回值：无
]]
local function proc(data)
    if not data or string.len(data) == 0 then return end
    rdbuf = rdbuf..data
    sys.publish("UART_RECEIVE")
    if collect then log.info("uart.buf","buff now",rdbuf:toHex()) return end
    if lock then return end
    if rdbuf:find("AT%+RELAY") then--继电器测试
        lock = true
        sys.taskInit(function ()
            pincfg.closeCheck(2000)
            local result = pincfg.relayStart(1,1)
            if result then
                result = sys.waitUntil("PLUSE_SENT",2000)--超时时间
                sys.wait(1000)--等待一秒，等继电器关上
            end
            if result then--成功
                write("AT+RELAY\r\n\r\nOK\r\n\r\n")
            else--失败
                write("AT+RELAY\r\n\r\nERROR\r\n\r\n")
            end
            lock = false
        end)
    elseif rdbuf:find("AT%+PLUSE") then--脉冲测试
        lock = true
        sys.taskInit(function ()
            local delay = uartPluse.sendPluse(50,50,1)
            pincfg.closeCheck(delay)
            local result = sys.waitUntil("PLUSE_SENT",2000)--超时时间
            if result then--成功
                write("AT+PLUSE\r\n\r\nOK\r\n\r\n")
            else--失败
                write("AT+PLUSE\r\n\r\nERROR\r\n\r\n")
            end
            lock = false
        end)
    elseif rdbuf:find("ATI") then--获取版本
        write("ATI\r\n\r\n"..rtos.get_version().."-".._G.PROJECT.."-".._G.VERSION.."\r\n\r\nOK\r\n\r\n")
    elseif rdbuf:find("AT%+CPIN") then--SIM是否在位
        if sim.getStatus() then
            write("AT+CPIN?\r\n\r\n+CPIN: READY\r\n\r\nOK\r\n\r\n")
        else
            write("AT+CPIN?\r\n\r\n+CME ERROR: 10\r\n\r\n")
        end
    elseif rdbuf:find("AT%+CESQ") then--当前CSQ
        lock = true
        ril.regUrc("+CESQ", function (data, prefix)
            write("AT+CESQ\r\n\r\n"..data.."\r\n\r\nOK\r\n\r\n")
            ril.deRegUrc("+CESQ")
            lock = false
        end)
        ril.request("AT+CESQ")
    elseif rdbuf:find("AT%+CGSN") then--是否注册上网络
        if misc.getImei() == "" then
            write("AT+CGSN\r\n\r\nERROR\r\n\r\n")
        else
            write("AT+CGSN\r\n\r\n"..misc.getImei().."\r\n\r\nOK\r\n\r\n")
        end
    elseif rdbuf:find("AT%+MUID") then--读取muid
        if misc.getMuid() == "" then
            write("AT+MUID?\r\n\r\nERROR\r\n\r\n")
        else
            write("AT+MUID?\r\n\r\n"..[[+MUID: "]]..misc.getMuid()..[["]].."\r\n\r\nOK\r\n\r\n")
            nvm.set("passCheck",true)
        end
    elseif rdbuf:sub(1,2) == string.fromHex("ef02") and rdbuf:len() == 5 then
        local sum = 0
        for i=1,rdbuf:len()-1 do
            sum = sum + rdbuf:byte(i)
        end
        if sum%256 == rdbuf:byte(-1) then--校验值匹配
            sys.publish("UART_REPLY",rdbuf:sub(3,3) == string.fromHex("01"))
        end
    elseif rdbuf == ("E77E01AA323CC3"):fromHex() then
        urlSent = true
    elseif receiveData and rdbuf == receiveData then--匹配返回值
        sys.publish("UART_RECEIVE_SET")
    else
        if not collect then
            sys.timerStart(function ()
                log.info("uart.buf","clear buf")
                rdbuf = ""
            end,500)
        end
        return
    end
    rdbuf = ""
end

--[[
函数名：read
功能  ：读取串口接收到的数据
参数  ：无
返回值：无
]]
local function read()
    local data = ""
    --底层core中，串口收到数据时：
    --如果接收缓冲区为空，则会以中断方式通知Lua脚本收到了新数据；
    --如果接收缓冲器不为空，则不会通知Lua脚本
    --所以Lua脚本中收到中断读串口数据时，每次都要把接收缓冲区中的数据全部读出，这样才能保证底层core中的新数据中断上来，此read函数中的while语句中就保证了这一点
    while true do
        data = uart.read(UART_ID,"*l")
        if not data or string.len(data) == 0 then break end
        log.info("uartOut.read hex",data:toHex())
        proc(data)
    end
end

--[[
函数名：write
功能  ：通过串口发送数据
参数  ：
        s：要发送的数据
返回值：无
]]
function write(s)
    pm.wake("uartOut")
    log.info("uartOut.write",s:toHex(),s)
    uart.write(UART_ID,s)
    -- if nvm.get("passCheck") ~= nil then
    --     sys.timerStart(pm.sleep,1000,"uartOut")
    -- end
end


--保持系统处于唤醒状态，此处只是为了测试需要，所以此模块没有地方调用pm.sleep("testUart")休眠，不会进入低功耗休眠状态
--在开发“要求功耗低”的项目时，一定要想办法保证pm.wake("testUart")后，在不需要串口时调用pm.sleep("testUart")
if nvm.get("passCheck") == nil then--如果没有测试过，就开机就开启唤醒
    local times = nvm.get("startTimes")
    if times == nil or times < 5 then--开机五次以内才开启唤醒
        pm.wake("uartOut")
        sys.taskInit(function ()--开机十分钟后永久不再始终唤醒
            sys.wait(10*60*1000)
            --pm.sleep("uartOut")
            nvm.set("startTimes",5)
            nvm.set("passCheck",true)
            log.info("startTimes",5,"exit wakeup mode forever!")
        end)
    else
        nvm.set("passCheck",true)--设置通过检测
    end

    if times == nil then
        nvm.set("startTimes",1)
    else
        nvm.set("startTimes",times+1)
    end
    log.info("startTimes",nvm.get("startTimes"))
end

--注册串口的数据接收函数，串口收到数据后，会以中断方式，调用read接口读取数据
uart.on(UART_ID,"receive",read)

--配置并且打开串口
uart.setup(UART_ID,9600,8,uart.PAR_NONE,uart.STOP_1)

--当前波特率
local uartBand = 9600
--更改波特率
function changeBand(band)
    log.info("uartOut.changeBand",uartBand,band)
    if band ~= uartBand then
        log.info("uartOut.changeBand","change to",band)
        uart.close(UART_ID)
        uart.on(UART_ID,"receive",read)
        uart.setup(UART_ID,tonumber(band),8,uart.PAR_NONE,uart.STOP_1)
        uartBand = band
    end
end
